library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_ARITH.all;
use IEEE.STD_LOGIC_UNSIGNED.all;
library UNISIM;
use UNISIM.vcomponents.all;


entity counter is
  port (clk50        : in  std_logic;
        sw_rst       : in  std_logic;
        sw0          : in  std_logic;
        sw1          : in  std_logic;
        sw2          : in  std_logic;
        sw3          : in  std_logic;
        jp0          : in  std_logic;
        headerout    : out std_logic;
        clk50_out    : out std_logic;
        fx2_clk60_p  : out std_logic;   -- differential outputs
        fx2_clk60_n  : out std_logic;
        fx2_outbit_p : out std_logic;
        fx2_outbit_n : out std_logic
        );
end counter;

architecture Behavioral of counter is

  component dcm1
    port(
      CLKIN_IN        : in  std_logic;
      RST_IN          : in  std_logic;  -- external positive reset signal
      CLKFX_OUT       : out std_logic;
      CLKIN_IBUFG_OUT : out std_logic;
      CLK0_OUT        : out std_logic;
      LOCKED_OUT      : out std_logic
      );
  end component;

  signal clk60     : std_logic;
  signal clk50_tmp : std_logic;
  signal input_bit : std_logic;

  signal count_int : std_logic_vector(31 downto 0) := x"00000000";
  signal bit_no    : std_logic_vector(5 downto 0);
  signal sendtrig  : std_logic_vector(10 downto 0);

  signal sr : std_logic_vector(37 downto 0) := "00" & x"000000000";

  signal rst_n   : std_logic;           -- active low reset
  signal sw0_val : std_logic;           -- active high, trigger bit 0
  signal sw1_val : std_logic;           -- active high, trigger bit 1
  signal sw2_val : std_logic;           -- active high, trigger bit 2
  signal sw3_val : std_logic;           -- active high, TDC reset on/off
  signal jp0_val : std_logic;  -- active high, event number counter on/off

  signal trig0    : std_logic;
  signal trig1    : std_logic;
  signal trig2    : std_logic;
  signal tdcreset : std_logic;
  
begin

  rst_n     <= not sw_rst;
  clk50_out <= clk50_tmp;               -- for single output on 6-pin header
  sw0_val   <= sw0;
  sw1_val   <= sw1;
  sw2_val   <= sw2;
  sw3_val   <= sw3;
  jp0_val   <= jp0;

  process (clk60, rst_n)
  begin  -- process
    if rst_n = '0' then                 -- asynchronous reset (active low)
      count_int <= (others => '0');
      bit_no    <= (others => '0');
      sr        <= (others => '0');
      sendtrig  <= (others => '0');
    elsif clk60'event and clk60 = '0' then  -- falling clk60 edge
      case sw0_val is
        when '1'    => trig0 <= '1';
        when '0'    => trig0 <= '0';
        when others => null;
      end case;
      case sw1_val is
        when '1'    => trig1 <= '1';
        when '0'    => trig1 <= '0';
        when others => null;
      end case;
      case sw2_val is
        when '1'    => trig2 <= '1';
        when '0'    => trig2 <= '0';
        when others => null;
      end case;
      case sw3_val is
        when '1'    => tdcreset <= '1';
        when '0'    => tdcreset <= '0';
        when others => null;
      end case;
      if bit_no = 37 then
        bit_no <= (others => '0');
        if sendtrig = 27 then           -- ~17 microseconds
          headerout <= '1';
          sendtrig  <= (others => '0');
          case jp0_val is
            when '1'    => sr <= "11" & tdcreset & trig2 & trig1 & trig0 & count_int;
            when '0'    => sr <= "11" & tdcreset & trig2 & trig1 & trig0 & x"00000000";
            when others => sr <= "11" & tdcreset & trig2 & trig1 & trig0 & x"00000000";
          end case;
          count_int <= count_int + 1;
        else
          sr       <= "000000" & x"00000000";
          sendtrig <= sendtrig + 1;
        end if;
      else
        headerout       <= '0';
        bit_no          <= bit_no + 1;
        input_bit       <= sr(37);      -- input to OBUFDS for differential out
        sr(37 downto 1) <= sr(36 downto 0);
      end if;
    end if;
  end process;


  dcm1_1 : dcm1
    port map (
      CLKIN_IN        => clk50,
      RST_IN          => sw_rst,
      CLKFX_OUT       => clk60,
      CLKIN_IBUFG_OUT => open,
      CLK0_OUT        => clk50_tmp,
      LOCKED_OUT      => open
      );

  OBUFDS_outbit : OBUFDS
    generic map (
      IOSTANDARD => "LVDS_25"
      )
    port map (
      O  => fx2_outbit_P,
      OB => fx2_outbit_n,
      I  => input_bit
      );

  OBUFDS_clk60 : OBUFDS
    generic map (
      IOSTANDARD => "LVDS_25"
      )
    port map (
      O  => fx2_clk60_p,
      OB => fx2_clk60_n,
      I  => clk60
      );

end Behavioral;


